package notio;

import java.io.*;

    /** 
     * The Generator interface.
     * All generators should implement this interface
     * to facilitate pluggability.  While a generator must implement
     * the entire interface, it can opt to throw an
     * UnimplementedFeature exception if it does not support generating
     * of a particular nonterminal. Minimally, a generator should support
     * the Unit nonterminal.
     *
     * @author Finnegan Southey
     * @version $Name:  $ $Revision: 1.14 $, $Date: 1999/05/04 01:35:51 $
		 * @legal Copyright (c) Finnegan Southey, 1996-1999
		 *	This program is free software; you can redistribute it and/or modify it 
		 *	under the terms of the GNU Library General Public License as published 
		 *	by the Free Software Foundation; either version 2 of the License, or 
		 *	(at your option) any later version.  This program is distributed in the 
		 *	hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
		 *	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
		 *	See the GNU Library General Public License for more details.  You should 
		 *	have received a copy of the GNU Library General Public License along 
		 *	with this program; if not, write to the Free Software Foundation, Inc., 
		 *	675 Mass Ave, Cambridge, MA 02139, USA.
     * @unfinished What methods should generators be forced to implement.
     * @idea Add OutermostContext() or TopLevelGraph()?  This would allow for automatic 
     * clearing of def. label tables, etc.
     */
public interface Generator
  {
    /**
     * Initializes the generator to generate into the specified writer.
     * 
     * @param writer  the writer to be generated into.
     * @param kBase  the knowledge base to be used while parsing.
     * @param tContext  the translation context to be used while parsing.
     * @exception GeneratorException  if an error occurs while initializing
     * the generator.
     */
  public void initializeGenerator(Writer writer, KnowledgeBase kBase,
  	TranslationContext tContext) throws GeneratorException;

    /**
     * Returns a Class object that indicates what class the Unit generate
     * method requires.
     * 
     * @return a Class object that indicates what class the Unit generate
     * method requires.
     * @see notio.Generator#Unit
     */
	public Class getUnitClass();

    /**
     * Generates the specified unit object.
     * 
     * @param unit  the unit to be generated.
     * @exception GeneratorException  if an error occurs while generating.
     */
  public void Unit(Object unit) throws GeneratorException;

    /**
     * Generates the specified graph.
     * 
     * @param graph  the graph to be generated.
     * @exception GeneratorException  if an error occurs while generating.
     */
  public void Graph(Graph graph) throws GeneratorException;

    /**
     * Generates the specified concept.
     * 
     * @param concept  the concept to be generated.
     * @exception GeneratorException  if an error occurs while generating.
     */
  public void Concept(Concept concept) throws GeneratorException;

    /**
     * Generates the specified relation.
     * 
     * @param relation  the relation to be generated.
     * @exception GeneratorException  if an error occurs while generating.
     */
  public void Relation(Relation relation) throws GeneratorException;
  }
