package notio.test;

import java.io.*;
import java.util.*;
import notio.*;
import notio.translators.*;

    /** 
     * Class used to test the CGIF translators.
     *
     * @author Finnegan Southey
     * @version $Name:  $ $Revision: 1.2 $, $Date: 2001/07/03 09:28:57 $
		 * @legal Copyright (c) Finnegan Southey, 1996-1999
		 *	This program is free software; you can redistribute it and/or modify it 
		 *	under the terms of the GNU Library General Public License as published 
		 *	by the Free Software Foundation; either version 2 of the License, or 
		 *	(at your option) any later version.  This program is distributed in the 
		 *	hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
		 *	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
		 *	See the GNU Library General Public License for more details.  You should 
		 *	have received a copy of the GNU Library General Public License along 
		 *	with this program; if not, write to the Free Software Foundation, Inc., 
		 *	675 Mass Ave, Cambridge, MA 02139, USA.
     */
public class TestCGIF extends TesterBase
  { 	
  	/** Flag indicating whether or not this test has passed. **/
  private boolean passed;
  
  	/**
  	 * Standard test access method.
  	 *
  	 * @return true if test was passed, false otherwise.
  	 */
  public boolean runTest()
  	{
		KnowledgeBase kBase;
		TranslationContext context;

		kBase = new KnowledgeBase();
		context = new TranslationContext();
		passed = true;

		if (false)
			{
			if (parseCGStreamFromFile(kBase, context, "ttt.cgf") == null)
				{
				logMessage("Failed to parse test graph.");
				passed = false;
				}
			}

		if (true)
			{	
			if (parseCGStreamFromString(kBase, context, TEST_GRAPH_1) == null)
				{
				logMessage("Failed to parse level 1 graphs.");
				passed = false;
				}
	
			if (parseCGStreamFromString(kBase, context, TEST_GRAPH_2) == null)
				{
				logMessage("Failed to parse level 2 graphs.");
				passed = false;
				}
	
			if (parseCGStreamFromString(kBase, context, TEST_GRAPH_3) == null)
				{
				logMessage("Failed to parse level 3 graphs.");
				passed = false;
				}
				
			if (parseCGStreamFromString(kBase, context, TEST_GRAPH_4) == null)
				{
				logMessage("Failed to parse level 4 graphs.");
				passed = false;
				}
	
			if (parseCGStreamFromString(kBase, context, TEST_GRAPH_5) == null)
				{
				logMessage("Failed to parse level 5 graphs.");
				passed = false;
				}
		}
			
/*
		if (parseCGStreamFromFile(kBase, context, "final_graphs_level1.cgf") == null)
			{
			logMessage("Failed to parse level 1 graphs.");
			passed = false;
			}

		if (parseCGStreamFromFile(kBase, context, "final_graphs_level2.cgf") == null)
			{
			logMessage("Failed to parse level 2 graphs.");
			passed = false;
			}

		if (parseCGStreamFromFile(kBase, context, "final_graphs_level3.cgf") == null)
			{
			logMessage("Failed to parse level 3 graphs.");
			passed = false;
			}
			
		if (parseCGStreamFromFile(kBase, context, "final_graphs_level4.cgf") == null)
			{
			logMessage("Failed to parse level 4 graphs.");
			passed = false;
			}

		if (parseCGStreamFromFile(kBase, context, "final_graphs_level5.cgf") == null)
			{
			logMessage("Failed to parse level 5 graphs.");
			passed = false;
			}
*/
		return passed;
  	}

		/**
  	 * Opens a file for reading and then attempts to parse a CGStream from it.
  	 *
  	 * @param kBase  the knowledge base that will be used during parsing.
  	 * @param tContext  the translation context that will be used during parsing.
  	 * @param str  the string to be parsed.
  	 * @return the parsed stream or null if the parsing fails for any reason.
  	 */
  public Vector parseCGStreamFromString(KnowledgeBase kBase, TranslationContext tContext,
  	String str)
  	{
  	StringReader reader = null;
  	CGIFParser parser;
  	Vector cgs;

		// INITIALIZING A READER
		// Any Reader can be used with the CGIFParser.  In this case we 
		// use a StringReader to the specified string.
		reader = new StringReader(str);
			
		// CREATING A PARSER
		// We now have all we need to create a CGIFParser instance.
		parser = new CGIFParser();
		try
			{
			parser.initializeParser(reader, kBase, tContext);
			}
	  catch (ParserException e)
	  	{
	  	e.printStackTrace();
	  	System.exit(1);
	  	}

		// PARSING A GRAPH FROM THE STREAM
		try
			{
			// Here, we ask the parser to try to parse a CGStream.  The CGIF parser
			// actually allows any non-terminal of the grammar to be parsed so
			// we could have asked for a Concept, Relation, or any other structure
			// instead.
			cgs = parser.parseCGStream();
			}
		catch (notio.ParserException e)
			{
			// This is the proper Notio exception that all parsers should throw
			// when they encounter problems.
			
			// Report error to user.
			logMessage(e.getMessage());
			return null;
			}
			
		return cgs;
  	}
		
		/**
  	 * Opens a file for reading and then attempts to parse a CGStream from it.
  	 *
  	 * @param kBase  the knowledge base that will be used during parsing.
  	 * @param tContext  the translation context that will be used during parsing.
  	 * @param filename  the name of the file to be opened.
  	 * @return the parsed stream or null if the parsing fails for any reason.
  	 */
  public Vector parseCGStreamFromFile(KnowledgeBase kBase, TranslationContext tContext,
  	String filename)
  	{
  	FileReader reader = null;
  	CGIFParser parser;
  	Vector cgs;

		// INITIALIZING A READER
		// Any Reader can be used with the CGIFParser.  In this case we 
		// use a FileReader to read from the file specified by filename.
		try
			{
			reader = new FileReader(filename);
			}
		catch (FileNotFoundException e)
			{
			// Report error to user.
			logMessage(e.getMessage());
			return null;
			}
			
		// CREATING A PARSER
		// We now have all we need to create a CGIFParser instance.
		parser = new CGIFParser();
		try
			{
			parser.initializeParser(reader, kBase, tContext);
			}
	  catch (ParserException e)
	  	{
			logMessage(e.getMessage());
	  	e.printStackTrace();
	  	}

		// PARSING A GRAPH FROM THE STREAM
		try
			{
			// Here, we ask the parser to try to parse a CGStream.  The CGIF parser
			// actually allows any non-terminal of the grammar to be parsed so
			// we could have asked for a Concept, Relation, or any other structure
			// instead.
			cgs = parser.parseCGStream();
			}
		catch (notio.ParserException e)
			{
			// This is the proper Notio exception that all parsers should throw
			// when they encounter problems.
			
			// Report error to user.
			logMessage(e.getMessage());
			return null;
			}

		try
			{
			reader.close();
			}
		catch (IOException e)
			{
			// Report error to user.
			logMessage(e.getMessage());
			return null;
			}
			
		return cgs;
  	}


  public String getTestName()
  	{
  	return "CGIF Test";
  	}
  	
  	/**
  	 * A main method so this test can be run independently as an application.
  	 *
  	 * @param args  the command line arguments.
  	 */
  public static void main(String args[])
  	{
  	TestCGIF test;
  	
  	test = new TestCGIF();
  	test.runAndReport();
  	}  	
	
	public final static String TEST_GRAPH_1 =
		"[Entity:'Block'\n" +
		"    (ATTR [Block*b] [Color])\n" +
		"    (CHRC ?b [Shape])\n" +
		" ;Each block has a color and shape; ]\n" +
		".\n" +
		"[Act:'Pickup'\n" +
		"    (PTNT [Pickup*p] [Block*b])\n" +
		"    (INST ?p [Hand*h])\n" +
		"    (RSLT ?p [Situation: (GRASP ?h ?b)])\n" +
		" ;Each block is picked up using a hand; ]\n" +
		".\n" +
		"[Act:'Putdown'\n" +
		"    (PTNT [Putdown*p] [Block*b])\n" +
		"    (DEST ?p [Location*l])\n" +
		"    (INST ?p [Hand])\n" +
		"    (RSLT ?p [Situation: (Top ?b ?l)])\n" +
		" ;Each block is put down at a location from the hand; ]\n" +
		".\n" +
		"[Act:'MoveHand'\n" +
		"    (DEST [MoveHand*m] [Location*l])\n" +
		"    (PTNT ?m [Hand*h])\n" +
		"    (RSLT ?m [Situation: (At ?h ?l)])\n" +
		" ;This action moves the hand to a location; ]\n" +
		".\n" +
		"[Act:'MoveBlock'\n" +
		"    (DEST [MoveBlock*m] [Location*l])\n" +
		"    (PTNT ?m [Block*b])\n" +
		"    (INST ?m [Hand])\n" +
		"    (RSLT ?m [Situation: (At ?b ?l)])\n" +
		" ;This action moves the block to a location; ]\n" +
		".\n" +
		"[Relation:'At'\n" +
		"    (POS [Entity] [Location])\n" +
		" ;An entity is positioned at a location; ]\n" +
		".\n" +
		"[Relation:'Top'\n" +
		"    (OnTable [Block*b1] [Location])\n" +
		"    ~[(Above [Block*b2] ?b1)]\n" +
		" ;A block on top is at a location and has no blocks above it; ]\n" +
		".\n" +
		"[Relation:'EmptyHand'\n" +
		"    ~[(GRASP [Hand] [Block])]\n" +
		" ;A hand is empty when no blocks are in it; ]\n" +
		".\n" +
		"[Relation:'OnTable'\n" +
		"    (At [Block*b] [Location])\n" +
		"    ~[(GRASP [Hand] ?b)]\n" +
		" ;A block on the table is at a location and not in the hand; ]\n" +
		".\n" +
		"[Relation:'Above'\n" +
		"    (OnTable [Block*b1] [Location*l])\n" +
		"    (OnTable [Block*b2] ?l)\n" +
		" ;The first block is above the second block at the same location; ]\n" +
		".\n" +
		";Block #1 is red and cubical;\n" +
		"(ATTR [Block*b:#1] [Color:'Red'])\n" +
		"(CHRC ?b [Shape:'Cubical']) \n" +
		".\n" +
		";Block #2 is Blue and cubical;\n" +
		"(ATTR [Block*b:#2] [Color:'Blue'])\n" +
		"(CHRC ?b [Shape:'Cubical']) \n" +
		".\n" +
		";Block #3 is green and cubical;\n" +
		"(ATTR [Block*b:#3] [Color:'Green'])\n" +
		"(CHRC ?b [Shape:'Cubical']) \n" +
		".\n" +
		"(OnTable [Block:#1] [Location:#5])\n" +
		".\n" +
		"(OnTable [Block:#2] [Location:#6])\n" +
		".\n" +
		"(OnTable [Block:#3] [Location:#5])\n" +
		".\n" +
		"(Above [Block:#1] [Block:#3])\n" +
		".\n" +
		";All the blocks are on the table and not in the hand;\n" +
		"(Emptyhand [Hand:#4])\n" +
		"";

	public final static String TEST_GRAPH_2 =
		"(GT [TypeLabel: \"Entity\"] [TypeLabel: \"Block\"]\n" +
		"    ;A block is an entity; )\n" +
		".\n" +
		"(GT [TypeLabel: \"Entity\"] [TypeLabel: \"Hand\"]\n" +
		"    ;A hand is an entity; )\n" +
		".\n" +
		"(GT [TypeLabel: \"Entity\"] [TypeLabel: \"Location\"]\n" +
		"    ;A location is an entity; )\n" +
		".\n" +
		"(GT [TypeLabel: \"Act\"] [TypeLabel: \"Pickup\"]\n" +
		"    ;Pickup is an action; )\n" +
		".\n" +
		"(GT [TypeLabel: \"Act\"] [TypeLabel: \"Putdown\"]\n" +
		"    ;Putdown is an action; )\n" +
		".\n" +
		"(GT [TypeLabel: \"Act\"] [TypeLabel: \"MoveHand\"]\n" +
		"    ;MoveHand is an action; )\n" +
		".\n" +
		"(GT [TypeLabel: \"Act\"] [TypeLabel: \"MoveBlock\"]\n" +
		"    ;MoveBlock is an action; )\n" +
		".\n" +
		"[Entity:'Block'\n" +
		"    (ATTR [Block*b] [Color])\n" +
		"    (CHRC ?b [Shape])\n" +
		" ;Each block has a color and shape; ]\n" +
		".\n" +
		"[Act:'Pickup'\n" +
		"    (PTNT [Pickup*p] [Block*b])\n" +
		"    (INST ?p [Hand*h])\n" +
		"    (RSLT ?p [Situation: (GRASP ?h ?b)])\n" +
		" ;Each block is picked up using a hand; ]\n" +
		".\n" +
		"[Act:'Putdown'\n" +
		"    (PTNT [Putdown*p] [Block*b])\n" +
		"    (DEST ?p [Location*l])\n" +
		"    (INST ?p [Hand])\n" +
		"    (RSLT ?p [Situation: (Top ?b ?l)])\n" +
		" ;Each block is put down at a location from the hand; ]\n" +
		".\n" +
		"[Act:'MoveHand'\n" +
		"    (DEST [MoveHand*m] [Location*l])\n" +
		"    (PTNT ?m [Hand*h])\n" +
		"    (RSLT ?m [Situation: (At ?h ?l)])\n" +
		" ;This action moves the hand to a location; ]\n" +
		".\n" +
		"[Act:'MoveBlock'\n" +
		"    (DEST [MoveBlock*m] [Location*l])\n" +
		"    (PTNT ?m [Block*b])\n" +
		"    (INST ?m [Hand])\n" +
		"    (RSLT ?m [Situation: (At ?b ?l)])\n" +
		" ;This action moves the block to a location; ]\n" +
		".\n" +
		"(GT [RelationLabel: \"Relation\"] [RelationLabel: \"At\"]\n" +
		"    ; Relation At ;)\n" +
		".\n" +
		"(GT [RelationLabel: \"Relation\"] [RelationLabel: \"Above\"]\n" +
		"    ; Relation Above ;)\n" +
		".\n" +
		"(GT [RelationLabel: \"Relation\"] [RelationLabel: \"OnTable\"]\n" +
		"    ; Relation OnTable ;)\n" +
		".\n" +
		"(GT [RelationLabel: \"Relation\"] [RelationLabel: \"Top\"]\n" +
		"    ; Relation Top ;)\n" +
		".\n" +
		"(GT [RelationLabel: \"Relation\"] [RelationLabel: \"EmptyHand\"]\n" +
		"    ; Relation EmptyHand ;)\n" +
		".\n" +
		"[Relation:'At'\n" +
		"    (POS [Entity] [Location])\n" +
		" ;An entity is positioned at a location; ]\n" +
		".\n" +
		"[Relation:'Top'\n" +
		"    (OnTable [Block*b1] [Location])\n" +
		"    ~[(Above [Block*b2] ?b1)]\n" +
		" ;A block on top is at a location and has no blocks above it; ]\n" +
		".\n" +
		"[Relation:'EmptyHand'\n" +
		"    ~[(GRASP [Hand] [Block])]\n" +
		" ;A hand is empty when no blocks are in it; ]\n" +
		".\n" +
		"[Relation:'OnTable'\n" +
		"    (At [Block*b] [Location])\n" +
		"    ~[(GRASP [Hand] ?b)]\n" +
		" ;A block on the table is at a location and not in the hand; ]\n" +
		".\n" +
		"[Relation:'Above'\n" +
		"    (OnTable [Block*b1] [Location*l])\n" +
		"    (OnTable [Block*b2] ?l)\n" +
		" ;The first block is above the second block at the same location; ]\n" +
		".\n" +
		"[Block:#1]\n" +
		".\n" +
		"[Block:#2]\n" +
		".\n" +
		"[Block:#3]\n" +
		".\n" +
		"[Hand:#4]\n" +
		".\n" +
		"[Location:#5]\n" +
		".\n" +
		"[Location:#6]\n" +
		".\n" +
		"[Block:@3]\n" +
		".\n" +
		";Block #1 is red;\n" +
		"(ATTR [Block:#1] [Color:'Red'])\n" +
		".\n" +
		";Block #2 is blue;\n" +
		"(ATTR [Block:#2] [Color:'Blue'])\n" +
		".\n" +
		";Block #3 is green;\n" +
		"(ATTR [Block:#3] [Color:'Green'])\n" +
		".\n" +
		"(OnTable [Block:#1] [Location:#5])\n" +
		".\n" +
		"(OnTable [Block:#2] [Location:#6])\n" +
		".\n" +
		"(OnTable [Block:#3] [Location:#5])\n" +
		".\n" +
		";Block #1 is above block #3, and block #2 is at a different location;\n" +
		"(Above [Block:#1] [Block:#3])\n" +
		".\n" +
		";All the blocks are on the table and not in the hand;\n" +
		"(Emptyhand [Hand:#4])\n" +
		".\n" +
		"[Either: [Or: (ATTR [Block:#1] [Color:'Blue'])]\n" +
		"         [Or: (ATTR [Block:#2] [Color:'Blue'])]]\n" +
		".\n" +
		";All blocks are cubical;\n" +
		"(CHRC [Block:@every] [Shape:'Cubical']) \n" +
		"";

	public final static String TEST_GRAPH_3 =
		"[KnowledgeBase:\n" +
		"  [TypeHierarchy:\n" +
		"    (GT [TypeLabel: \"Entity\"] [TypeLabel: \"Block\"]\n" +
		"        ;A block is an entity; )\n" +
		"    (GT [TypeLabel: \"Entity\"] [TypeLabel: \"Hand\"]\n" +
		"        ;A hand is an entity; )\n" +
		"    (GT [TypeLabel: \"Entity\"] [TypeLabel: \"Location\"]\n" +
		"        ;A location is an entity; )\n" +
		"    (GT [TypeLabel: \"Act\"] [TypeLabel: \"Pickup\"]\n" +
		"        ;Pickup is an action; )\n" +
		"    (GT [TypeLabel: \"Act\"] [TypeLabel: \"Putdown\"]\n" +
		"        ;Putdown is an action; )\n" +
		"    (GT [TypeLabel: \"Act\"] [TypeLabel: \"MoveHand\"]\n" +
		"	;MoveHand is an action; )\n" +
		"    (GT [TypeLabel: \"Act\"] [TypeLabel: \"MoveBlock\"]\n" +
		"	;MoveBlock is an action; )\n" +
		"    (Def [TypeLabel: \"Block\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Entity*b)\n" +
		"                   (ATTR ?b [Color])\n" +
		"                   (CHRC ?b [Shape]))\" ]\n" +
		"     ;Definition that each BLOCK has a color and shape; )\n" +
		"    (Def [TypeLabel: \"Pickup\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Act*p)\n" +
		"                   (PTNT ?p [Block*b])\n" +
		"                   (INST ?p [Hand*h])\n" +
		"                   (RSLT ?p [Situation: (GRASP ?h ?b)]) )\" ]\n" +
		"     ;Definition that each block is PICKUP using a hand; )\n" +
		"    (Def [TypeLabel: \"Putdown\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Act*p)\n" +
		"                   (PTNT ?p [Block*b])\n" +
		"                   (DEST ?p [Location*l])\n" +
		"                   (INST ?p [Hand])\n" +
		"                   (RSLT ?p [Situation: (Top ?b ?l)]) )\" ]\n" +
		"     ;Definition that each block is PUTDOWN at a location from the hand; )\n" +
		"    (Def [TypeLabel: \"MoveHand\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Act*m)\n" +
		"                   (DEST ?m [Location*l])\n" +
		"                   (INST ?m [Hand*h])\n" +
		"                   (RSLT ?m [Situation: (At ?h ?l)]) )\" ]\n" +
		"     ;Definition of action that MOVEHAND to a location; )\n" +
		"    (Def [TypeLabel: \"MoveBlock\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Act*m)\n" +
		"                   (DEST ?m [Location*l])\n" +
		"                   (PTNT ?m [Block*b])\n" +
		"                   (INST ?m [Hand])\n" +
		"                   (RSLT ?m [Situation: (At ?b ?l)]) )\" ]\n" +
		"     ;Definition of action that MOVEBLOCK to a location;)\n" +
		"  ;Type Hierarchy defines some entities and actions and their type relationships;\n" +
		"  ]\n" +
		"  [RelationHierarchy:\n" +
		"    (GT [RelationLabel: \"Relation\"] [RelationLabel: \"At\"]\n" +
		"        ; Relation At ; )\n" +
		"    (GT [RelationLabel: \"Relation\"] [RelationLabel: \"Above\"]\n" +
		"        ; Relation Above ; )\n" +
		"    (GT [RelationLabel: \"Relation\"] [RelationLabel: \"OnTable\"]\n" +
		"        ; Relation OnTable ; )\n" +
		"    (GT [RelationLabel: \"Relation\"] [RelationLabel: \"Top\"]\n" +
		"        ; Relation Top ; )\n" +
		"    (GT [RelationLabel: \"Relation\"] [RelationLabel: \"EmptyHand\"]\n" +
		"        ; Relation EmptyHand ; )\n" +
		"    (Def [RelationLabel: \"At\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Entity*e1, Location*l)\n" +
		"                   (POS ?e1 ?l))\" ]\n" +
		"     ;Definition of an entity that is positioned AT a location; )\n" +
		"    (Def [RelationLabel: \"Top\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Block*b1,Location*l)\n" +
		"                   (OnTable ?b1 ?l)\n" +
		"                   ~[(Above [Block*b2] ?b1)])\" ]\n" +
		"     ;Definition of a block on TOP is at a location and has no blocks above it; )\n" +
		"    (Def [RelationLabel: \"EmptyHand\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Hand*h)\n" +
		"                   ~[(GRASP ?h [Block])])\" ]\n" +
		"     ;Definition of a EMPTYHAND is when no blocks are in it; )\n" +
		"    (Def [RelationLabel: \"OnTable\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Block*b,Location*l)\n" +
		"                   (At ?b ?l)\n" +
		"                   ~[(GRASP [Hand] ?b)])\" ]\n" +
		"     ;Definition of a block ONTABLE is at a location and not in the hand; )\n" +
		"    (Def [RelationLabel: \"Above\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Block*b1,Block*b2)\n" +
		"                   (OnTable ?b1 [Location*l])\n" +
		"                   (OnTable ?b2 ?l))\" ]\n" +
		"     ;Definition of the first block is ABOVE the second block at the same location; )\n" +
		"  ;Relation Hierarchy defines some new relations; \n" +
		"  ]\n" +
		"  [CatalogOfIndividuals:\n" +
		"    [Block:#1] [Block:#2] [Block:#3] [Hand:#4] [Location:#5] [Location:#6]\n" +
		"  ]\n" +
		"  [Assertion:\n" +
		"    [Block:@3]\n" +
		"    (ATTR [Block:#1] [Color:'Red'] ;Block #1 is red;)\n" +
		"    (ATTR [Block:#2] [Color:'Blue'] ;Block #2 is blue;)\n" +
		"    (ATTR [Block:#3] [Color:'Green'] ;Block #3 is green;)\n" +
		"    (OnTable [Block:#1] [Location:#5])\n" +
		"    (OnTable [Block:#2] [Location:#6])\n" +
		"    (OnTable [Block:#3] [Location:#5])\n" +
		"    (Above [Block:#1] [Block:#3] ;Block #1 is above block #3, and block #2 is at a different location;)\n" +
		"    (Emptyhand [Hand:#4] ;All the blocks are on the table and not in the hand;)\n" +
		"    [Either: [Or: (ATTR [Block:#1] [Color:'Blue'])]\n" +
		"             [Or: (ATTR [Block:#2] [Color:'Blue'])]]\n" +
		"    (CHRC [Block:@every] [Shape:'Cubical'] ;All blocks are cubical;)\n" +
		"  ;These are beginning fact graphs in the knowledge base;\n" +
		"  ]\n" +
		";This is a Blocks World KB.  It is not meant to be complete.;\n" +
		"]\n" +
		"";

	public final static String TEST_GRAPH_4 =
		"[KnowledgeBase:\n" +
		"  [TypeHierarchy:\n" +
		"    (GT [TypeLabel: \"Entity\"]\n" +
		"        [TypeLabel: @Col{\"Block\", \"Hand\", \"Location\"} ]\n" +
		"        ;block, hand, and location are entities; )\n" +
		"    (GT [TypeLabel: \"Act\"]\n" +
		"        [TypeLabel: @Col{\"Pickup\", \"Putdown\", \"MoveHand\", \"MoveBlock\"}]\n" +
		"        ;pickup, putdown, movehand, and moveblock are actions; )\n" +
		"    (Def [TypeLabel: \"Block\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Entity*b)\n" +
		"                   (ATTR ?b [Color])\n" +
		"                   (CHRC ?b [Shape]))\" ]\n" +
		"     ;Definition that each BLOCK has a color and shape; )\n" +
		"    (Def [TypeLabel: \"Pickup\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Act*p)\n" +
		"                   (PTNT ?p [Block*b])\n" +
		"                   (INST ?p [Hand*h])\n" +
		"                   (RSLT ?p [Situation: (GRASP ?h ?b)]) )\" ]\n" +
		"     ;Definition that each block is PICKUP using a hand; )\n" +
		"    (Def [TypeLabel: \"Putdown\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Act*p)\n" +
		"                   (PTNT ?p [Block*b])\n" +
		"                   (DEST ?p [Location*l])\n" +
		"                   (INST ?p [Hand])\n" +
		"                   (RSLT ?p [Situation: (Top ?b ?l)]) )\" ]\n" +
		"     ;Definition that each block is PUTDOWN at a location from the hand; )\n" +
		"    (Def [TypeLabel: \"MoveHand\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Act*m)\n" +
		"                   (DEST ?m [Location*l])\n" +
		"                   (INST ?m [Hand*h])\n" +
		"                   (RSLT ?m [Situation: (At ?h ?l)]) )\" ]\n" +
		"     ;Definition of action that MOVEHAND to a location; )\n" +
		"    (Def [TypeLabel: \"MoveBlock\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Act*m)\n" +
		"                   (DEST ?m [Location*l])\n" +
		"                   (PTNT ?m [Block*b])\n" +
		"                   (INST ?m [Hand])\n" +
		"                   (RSLT ?m [Situation: (At ?b ?l)]) )\" ]\n" +
		"     ;Definition of action that MOVEBLOCK to a location;)\n" +
		"  ;Type Hierarchy defines some entities and actions and their type relationships;\n" +
		"  ]\n" +
		"  [RelationHierarchy:\n" +
		"    (GT [RelationLabel: \"Relation\"]\n" +
		"        [RelationLabel:\n" +
		"               @Col{\"At\", \"Above\", \"OnTable\", \"Top\", \"EmptyHand\"}]\n" +
		"        ;New relations - at, above, ontable, top, and emptyhand; )\n" +
		"    (Def [RelationLabel: \"At\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Entity*e1, Location*l)\n" +
		"                   (POS ?e1 ?l))\" ]\n" +
		"     ;Definition of an entity that is positioned AT a location; )\n" +
		"    (Def [RelationLabel: \"Top\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Block*b1,Location*l)\n" +
		"                   (OnTable ?b1 ?l)\n" +
		"                   ~[(Above [Block*b2] ?b1)])\" ]\n" +
		"     ;Definition of a block on TOP is at a location and has no blocks above it; )\n" +
		"    (Def [RelationLabel: \"EmptyHand\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Hand*h)\n" +
		"                   ~[(GRASP ?h [Block])])\" ]\n" +
		"     ;Definition of a EMPTYHAND is when no blocks are in it; )\n" +
		"    (Def [RelationLabel: \"OnTable\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Block*b,Location*l)\n" +
		"                   (At ?b ?l)\n" +
		"                   ~[(GRASP [Hand] ?b)])\" ]\n" +
		"     ;Definition of a block ONTABLE is at a location and not in the hand; )\n" +
		"    (Def [RelationLabel: \"Above\"]\n" +
		"         [LambdaExpression:\n" +
		"            \"(lambda (Block*b1,Block*b2)\n" +
		"                   (OnTable ?b1 [Location*l])\n" +
		"                   (OnTable ?b2 ?l))\" ]\n" +
		"     ;Definition of the first block is ABOVE the second block at the same location; )\n" +
		"  ;Relation Hierarchy defines some new relations; \n" +
		"  ]\n" +
		"  [CatalogOfIndividuals:\n" +
		"    [Block:#1] [Block:#2] [Block:#3] [Hand:#4] [Location:#5] [Location:#6]\n" +
		"  ]\n" +
		"  [Assertion:\n" +
		"    [Block:@3]\n" +
		"    (ATTR [Block:#1] [Color:'Red'] ;Block #1 is red;)\n" +
		"    (ATTR [Block:#2] [Color:'Blue'] ;Block #2 is blue;)\n" +
		"    (ATTR [Block:#3] [Color:'Green'] ;Block #3 is green;)\n" +
		"    (OnTable [Block:#1] [Location:#5])\n" +
		"    (OnTable [Block:#2] [Location:#6])\n" +
		"    (OnTable [Block:#3] [Location:#5])\n" +
		"    (Above [Block:#1] [Block:#3] ;Block #1 is above block #3, and block #2 is at a different location;)\n" +
		"    (Emptyhand [Hand:#4] ;All the blocks are on the table and not in the hand;)\n" +
		"    [Either: [Or: (ATTR [Block:#1] [Color:'Blue'])]\n" +
		"             [Or: (ATTR [Block:#2] [Color:'Blue'])]]\n" +
		"    (CHRC [Block:@every] [Shape:'Cubical'] ;All blocks are cubical;)\n" +
		"  ;These are beginning fact graphs in the knowledge base;\n" +
		"  ]\n" +
		";This is a Blocks World KB.  It is not meant to be complete.;\n" +
		"]\n" +
		"";

	public final static String TEST_GRAPH_5 =
		"[KnowledgeBase:[TypeHierarchy:(GT[TypeLabel:\"Entity\"][TypeLabel:@Col{\"Block\",\"Hand\",\"Location\"}])(GT[TypeLabel:\"Act\"][TypeLabel:@Col{\"Pickup\",\"Putdown\",\"Move\"}])(Def[TypeLabel:\"Block\"][LambdaExpression:\"(lambda(Entity*b)(ATTR?b[Color])(CHRC?b[Shape]))\"])(Def[TypeLabel:\"Pickup\"][LambdaExpression:\"(lambda(Act*p)(PTNT?p[Block*b])(INST?p[Hand*h])(RSLT?p[Situation:(GRASP?h?b)]))\"])(Def[TypeLabel:\"Putdown\"][LambdaExpression:\"(lambda(Act*p)(PTNT?p[Block*b])(DEST?p[Location*l])(INST?p[Hand])(RSLT?p[Situation:(Top?b?l)]))\"])(Def[TypeLabel:\"MoveHand\"][LambdaExpression:\"(lambda(Act*m)(DEST?m[Location*l])(INST?m[Hand*h])(RSLT?m[Situation:(At?h?l)]))\"])(Def[TypeLabel:\"MoveBlock\"][LambdaExpression:\"(lambda(Act*m)(DEST?m[Location*l])(PTNT?m[Block*b])(INST?m[Hand])(RSLT?m[Situation:(At?b?l)]))\"])][RelationHierarchy:(GT[RelationLabel:\"Relation\"][RelationLabel:@Col{\"At\",\"Above\",\"OnTable\",\"Top\",\"EmptyHand\"}])(Def[RelationLabel:\"At\"][LambdaExpression:\"(lambda(Entity*e1,Location*l)(POS?e1?l))\"])(Def[RelationLabel:\"Top\"][LambdaExpression:\"(lambda(Block*b1,Location*l)(OnTable?b1?l)~[(Above[Block*b2]?b1)])\"])(Def[RelationLabel:\"EmptyHand\"][LambdaExpression:\"(lambda(Hand*h)~[(GRASP?h[Block])])\"])(Def[RelationLabel:\"OnTable\"][LambdaExpression:\"(lambda(Block*b,Location*l)(At?b?l)~[(GRASP[Hand]?b)])\"])(Def[RelationLabel:\"Above\"][LambdaExpression:\"(lambda(Block*b1,Block*b2)(OnTable?b1[Location*l])(OnTable?b2?l))\"])][CatalogOfIndividuals:[Block:#1][Block:#2][Block:#3][Hand:#4][Location:#5][Location:#6]][Assertion:[Block:@3](ATTR[Block:#1][Color:'Red'])(ATTR[Block:#2][Color:'Blue'])(ATTR[Block:#3][Color:'Green'])(OnTable[:#1][:#5])(OnTable[:#2][:#6])(OnTable[:#3][:#5])(Above[:#1][:#3])(Emptyhand[:#4])[Either:[Or:(ATTR[Block:#1][Color:'Blue'])][Or:(ATTR[Block:#2][Color:'Blue'])]](CHRC[Block:@every][Shape:'Cubical'])]]\n" +		"";
	}
