package notio.test;

import notio.*;

    /** 
     * Class used to test the various matching methods.
     *
     * @author Finnegan Southey
     * @version $Name:  $ $Revision: 1.3 $, $Date: 2001/07/03 12:27:42 $
		 * @legal Copyright (c) Finnegan Southey, 1996-1999
		 *	This program is free software; you can redistribute it and/or modify it 
		 *	under the terms of the GNU Library General Public License as published 
		 *	by the Free Software Foundation; either version 2 of the License, or 
		 *	(at your option) any later version.  This program is distributed in the 
		 *	hope that it will be useful, but WITHOUT ANY WARRANTY; without even the 
		 *	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
		 *	See the GNU Library General Public License for more details.  You should 
		 *	have received a copy of the GNU Library General Public License along 
		 *	with this program; if not, write to the Free Software Foundation, Inc., 
		 *	675 Mass Ave, Cambridge, MA 02139, USA.
     */
public class TestMatch extends TesterBase
  { 	
  	/** Flag indicating whether or not this test has passed. **/
  private boolean passed;
  	/** Testing fixture. */
 	private TestFixtures.ConceptTypeHierarchyFixtureOne cHierFixture;
  	/** Testing fixture. */
 	private TestFixtures.RelationTypeHierarchyFixtureOne rHierFixture;
  	/** Testing fixture. */
 	private TestFixtures.SimpleGraphFixtureOne simpleGraphs1;
  	/** Testing fixture. */
 	private TestFixtures.SimpleGraphFixtureOne simpleGraphs2;
 		/** Pairs of graphs for each combination. */
 	private Graph graphList[];
 		/** Description for each combination. */
 	private String combinationList[];

		/**
		 * Tests a scheme by attempting to match several combinations of graphs and checking
		 * whether the expected number of matches is found in each case.
		 *
		 * @param scheme  the scheme to be tested.
		 * @param schemeName  a description to identify the scheme when logging results.
		 * @param expectedMatches  an array containing the expected number of matches for each
		 * combination.
		 */
	private void testScheme(MatchingScheme scheme, String schemeName, int expectedMatches[])
		{
  	MatchResult result;

  	for (int comb = 0; comb < combinationList.length; comb++)
  		{
			// Match two graphs
//			System.err.println("Matching: " + combinationList[comb]);
			result = Graph.matchGraphs(graphList[comb * 2], graphList[comb * 2 + 1], scheme);

			// Checked match result for correct number of matches.
			if (result.getNumberOfMatches() != expectedMatches[comb])
				{
				passed = false;
				logMessage("Bad match: scheme '" + schemeName + "' on '" + combinationList[comb] + "' produced: " + result.getNumberOfMatches() + " expected: " + expectedMatches[comb]);
/*				logMessage(MatchAnalyzer.analyzeMatchResult(result)); */
				}
			}
  	}
  	
  	/**
  	 * Standard test access method.
  	 *
  	 * @return true if test was passed, false otherwise.
  	 */
  public boolean runTest()
  	{
  	
		passed = true;
		
		// Construct fixtures
  	cHierFixture = new TestFixtures.ConceptTypeHierarchyFixtureOne();
  	rHierFixture = new TestFixtures.RelationTypeHierarchyFixtureOne();
		simpleGraphs1 = new TestFixtures.SimpleGraphFixtureOne(cHierFixture, rHierFixture);
		simpleGraphs2 = new TestFixtures.SimpleGraphFixtureOne(cHierFixture, rHierFixture);

		// Build combinations
		Graph gList[] = 
			{
			null, null,
			simpleGraphs1.firstSimpleGraph, null,
			null, simpleGraphs2.firstSimpleGraph,
			simpleGraphs1.firstSimpleGraph, simpleGraphs2.firstSimpleGraph,
			simpleGraphs1.secondSimpleGraph, simpleGraphs2.secondSimpleGraph,
			simpleGraphs1.secondSimpleGraph, simpleGraphs2.tenthSimpleGraph,
			simpleGraphs1.tenthSimpleGraph, simpleGraphs2.secondSimpleGraph,
			simpleGraphs1.sixthSimpleGraph, simpleGraphs2.secondSimpleGraph,
			simpleGraphs1.secondSimpleGraph, simpleGraphs2.sixthSimpleGraph,
			simpleGraphs1.seventhSimpleGraph, simpleGraphs2.secondSimpleGraph,
			simpleGraphs1.secondSimpleGraph, simpleGraphs2.seventhSimpleGraph,
			simpleGraphs1.ninthSimpleGraph, simpleGraphs2.eigthSimpleGraph,
			simpleGraphs1.eigthSimpleGraph, simpleGraphs2.ninthSimpleGraph,
			};
		
		graphList = gList;
			
		String cList[] = 
			{
			"null : null",
			"[] : null",
			"null : []",
			"[] : []",
			"[A] : [A]",
			"[A] : [B]",
			"[B] : [A]",
			"[A][A] : [A]",
			"[A] : [A][A]",
			"[A][A][A] : [A]",
			"[A] : [A][A][A]",
			"[A][A][B][B] : [A][B]",
			"[A][B] : [A][A][B][B]",
			};				

			
		combinationList = cList;
			
		// Build schemes
		MatchingScheme schemeList[] = 
			{
				// "projection/folds"
			new MatchingScheme(
				MatchingScheme.GR_MATCH_SUBGRAPH,
				MatchingScheme.CN_MATCH_ALL,
				MatchingScheme.RN_MATCH_ALL,
				MatchingScheme.CT_MATCH_SUBTYPE,
				MatchingScheme.RT_MATCH_SUBTYPE,
				MatchingScheme.QF_MATCH_ANYTHING,
				MatchingScheme.DG_MATCH_RESTRICTION,
				MatchingScheme.MARKER_MATCH_ID,
				MatchingScheme.ARC_MATCH_CONCEPT,
				MatchingScheme.COREF_AUTOMATCH_OFF,
				MatchingScheme.COREF_AGREE_OFF,
				MatchingScheme.FOLD_MATCH_ON,
				MatchingScheme.CONN_MATCH_OFF,
				0,
				null,
				null)
			,

			// "projection/no folds"
			new MatchingScheme(
				MatchingScheme.GR_MATCH_SUBGRAPH,
				MatchingScheme.CN_MATCH_ALL,
				MatchingScheme.RN_MATCH_ALL,
				MatchingScheme.CT_MATCH_SUBTYPE,
				MatchingScheme.RT_MATCH_SUBTYPE,
				MatchingScheme.QF_MATCH_ANYTHING,
				MatchingScheme.DG_MATCH_RESTRICTION,
				MatchingScheme.MARKER_MATCH_ID,
				MatchingScheme.ARC_MATCH_CONCEPT,
				MatchingScheme.COREF_AUTOMATCH_OFF,
				MatchingScheme.COREF_AGREE_OFF,
				MatchingScheme.FOLD_MATCH_OFF,
				MatchingScheme.CONN_MATCH_OFF,
				0,
				null,
				null)
			,
				
			// "complete/folds"
			new MatchingScheme(
				MatchingScheme.GR_MATCH_COMPLETE,
				MatchingScheme.CN_MATCH_ALL,
				MatchingScheme.RN_MATCH_ALL,
				MatchingScheme.CT_MATCH_SUBTYPE,
				MatchingScheme.RT_MATCH_SUBTYPE,
				MatchingScheme.QF_MATCH_ANYTHING,
				MatchingScheme.DG_MATCH_RESTRICTION,
				MatchingScheme.MARKER_MATCH_ID,
				MatchingScheme.ARC_MATCH_CONCEPT,
				MatchingScheme.COREF_AUTOMATCH_OFF,
				MatchingScheme.COREF_AGREE_OFF,
				MatchingScheme.FOLD_MATCH_ON,
				MatchingScheme.CONN_MATCH_OFF,
				0,
				null,
				null)
			,
				
			// "complete/no folds"
			new MatchingScheme(
				MatchingScheme.GR_MATCH_COMPLETE,
				MatchingScheme.CN_MATCH_ALL,
				MatchingScheme.RN_MATCH_ALL,
				MatchingScheme.CT_MATCH_SUBTYPE,
				MatchingScheme.RT_MATCH_SUBTYPE,
				MatchingScheme.QF_MATCH_ANYTHING,
				MatchingScheme.DG_MATCH_RESTRICTION,
				MatchingScheme.MARKER_MATCH_ID,
				MatchingScheme.ARC_MATCH_CONCEPT,
				MatchingScheme.COREF_AUTOMATCH_OFF,
				MatchingScheme.COREF_AGREE_OFF,
				MatchingScheme.FOLD_MATCH_OFF,
				MatchingScheme.CONN_MATCH_OFF,
				0,
				null,
				null)
			};
		
		String schemeNameList[] = 
			{
			"projection/folds",
			"projection/no folds",
			"complete/folds",
			"complete/no folds"
			};
			
		int expectedMatchesTable[][] = 
			{
				// matching scheme for "projection/folds"
				{ 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1 },
				// matching scheme for "projection/no folds"
				{ 1, 1, 1, 1, 1, 0, 0, 0, 2, 0, 3, 0, 4 },
				// matching scheme for "complete/folds"
				{ 1, 1, 1, 1, 1, 0, 0, 1, 1, 1, 1, 1, 1 },
				// matching scheme for "complete/no folds"
				{ 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0 }
			};
			
			
		int sch = 1;
//		for (int sch = 0; sch < schemeList.length; sch++)
			testScheme(schemeList[sch], schemeNameList[sch], expectedMatchesTable[sch]);
			
		return passed;
  	}

  public String getTestName()
  	{
  	return "Match Test";
  	}
  	
  	/**
  	 * A main method so this test can be run independently as an application.
  	 *
  	 * @param args  the command line arguments.
  	 */
  public static void main(String args[])
  	{
  	TestMatch test;
  	
  	test = new TestMatch();
  	test.runAndReport();
  	}  	
  }
